package CatalystX::state51::LegacySupport;
use MooseX::MethodAttributes::Role;
use Moose::Autobox;
use MooseX::Types::LoadableClass qw/ClassName/;
use state51::Base;
use Try::Tiny qw/ try catch /;
use namespace::autoclean;

our $VERSION = '0.011';

sub BUILD {}
before BUILD => sub {
    my $self = shift;
    state51::Base->connect;
};
after BUILD => sub {
    my $self = shift;
    $self->collective_id;
};

has collective_name => (
    is => 'ro',
    default => 'cisupport',
);

# Note - we grab the collective here and throw it away in favour of the ID
#        Loading by ID subsequent to the first time will come from memcache, so this wins..
has collective_id => (
    is => 'ro',
    lazy => 1,
    default => sub { (state51::Collective->fetch(name => shift->collective_name))[0]->id },
);

requires 'site_config';

sub base : Chained('/') PathPart('') CaptureArgs(0) {
    my ($self, $c) = @_;
    my $uri = $c->request->uri;
    $uri =~ s/\?.*$//;
    my $p = sub {
        my $param = shift;
        if ($param) {
            if ($param eq 'uri_params') {
                return @{ $c->stash->{uri_params} };
            }
            else {
                return $c->req->param($param);
            }
        }
        else {
            return $c->req->parameters->keys;
        }
    };
    {   # Pure evil here! We entirely replace the 'p' method in
        # state51::View::Base with another copy, which we can only safely do
        # as we run in our own process space.
        # FIXME - Gross hack!
        no strict 'refs';
        no warnings 'redefine';        # Throw away $self, call $p function
        no warnings 'once';            # with the rest of the args
        *state51::View::Base::p = sub { shift; $p->(@_); };
    }
     $c->stash(
        collective => state51::Collective->load($self->collective_id),
        site_config => $self->site_config,
        dwarn => sub {
            $c->log->warn(shift) if $c->debug;
        },
        uri => $uri,
        p => $p,
    );
    $state51::View::Base::LEGACY_WARNING = 1;
    if ($c->session->{_role_id}) { # FIXME - Do not create session if one doesn't exist
        try {
            my $role = state51::Role->load($c->session->{_role_id});
            $c->stash(
                data => { put__role => $role },
                role => $role,
                put_role => $role,
                is_user_admin => $role->has_authority('put system admin'),
            );
        }
        catch {
            $c->log->warn("Could not load Role from session, id: " . $c->session->{_role_id});
            delete($c->session->{_role_id});
        };
    }
}

sub default : Chained('base') PathPart('') Args {
    my ( $self, $c ) = @_;
    my @args = grep { length $_ } split /\//, $c->req->path;
    my @params = map { local $_ = $_; s/^~//; $_ } grep { /^~/ } @args;
    $c->stash( uri_params => \@params );
    my $path = join('/', grep { ! /^~/ } @args);

    # Ensure that there is a trailing / on directory links, otherwise
    # relative URIs don't work as expected.
    if ( -d $c->path_to('root', $path) and $c->req->uri->path !~ m{/$} ) {
        my $uri = $c->req->uri->clone;
        $uri->path($uri->path . '/');
        $c->res->redirect($uri);
        $c->res->status(301);
        $c->detach;
    }

    if ($path !~ /\w+\.\w+$/) {
        $path =~ s{^/}{};
        $path .= '/index.html';
    }
    $path =~ s{^/}{};
    $c->log->debug('Template path: /' . $path) if $c->debug;
    $c->detach('error404')
        unless -f $c->path_to('root', $path);
    $c->stash(template => $path);
    $state51::View::Base::LEGACY_WARNING = 0;
    if ($c->req->path =~ /\.csv$/ || $c->req->path =~ /~csv/) {
      my $filename = (split("\/",$c->req->path))[-1];
      $filename =~ s/html$/csv/;
      $filename .='.csv' if ($filename !~ m/\.csv$/);
      $c->response->headers->header("Content-Type" => "text/csv");
      $c->res->headers->header( 'Content-Disposition' =>
                                "attachment; filename=$filename");
    }
    if ($c->req->method eq 'POST') {
        $c->forward('default_POST');
    }
    elsif (!$self->idempotent_GET && $c->req->params->{'form_action'}) {
        $c->forward('default_POST');
    }
}

has idempotent_GET => (
    is => 'ro',
    default => 1,
);

has legacy_view_class => (
    is => 'ro',
    default => 'state51::View::Simple::Base',
    isa => ClassName,
    coerce => 1,
);

sub default_POST : Action {
    my ($self, $c) = @_;
    my @actions;
    foreach my $p (qw/form_action formaction act/) {
        push(@actions, $c->req->parameters->{$p}->flatten)
            if $c->req->parameters->{$p};
    }
    my $view_class = $self->legacy_view_class;
    my %p = (
        site_config => $c->stash->{site_config},
        data => $c->stash->{data} || {},
        session => $c->session,
        collective_name => $c->stash->{collective}->name,
        mail_extra_tt_includes => [ $c->path_to('mail') ],
    );
    if ($view_class->can('retailer')) {
        $p{retailer} = $c->stash->{retailer};
    }
    my $view = $c->stash->{__legacy_view} = $view_class->new(%p); # Note, we stash the view object, as it calls unload_all!
    {   # Pure evil here! We entirely replace the 'upload' method in
        # state51::View::Base with another copy, which we can only safely do
        # as we run in our own process space.
        # FIXME - Gross hack!
        no strict 'refs';
        no warnings 'redefine';            # Throw away $self, get upload
        no warnings 'once';                # with the upload field name
        *state51::View::Base::upload = sub { shift; $c->req->upload(shift); };
    }
    $view->{collective} = $c->stash->{collective}; # Hacky hack..
    my %munged_params = map {
        defined $c->req->parameters->{$_} ?
            ($_ => [$c->req->parameters->{$_}->flatten] )
            : ( $_ => [undef] )
    } keys(%{$c->req->parameters});
    
    # Add in request IP address - needed to persist with any new GreedOrder
    if(defined($c->req->address)) {
        $munged_params{'ip_address'} = [$c->req->address]; 
    }  
    
    $view->{_p} = \%munged_params;
    $view->role($c->stash->{role})
        if $c->stash->{role};
    foreach my $name (@actions) {
        my $code = $view_class->can('action__' . $name);
        die("Could not find \"action__$name\" in $view_class")
            unless $code;
        eval { $view->$code };
        if ($@) {
            my $error = $@;
            require state51::Dumper;

            die("Caught error in legacy handler code from request "
                . $c->req->method . " "
                . " to " . $c->req->uri . " params "
                . state51::Dumper::Dumper($c->req->parameters)
                . " message: $error");
        }
    }
    if ($view->role) {
        $c->session(_role_id => $view->role->id);
        $c->stash(role => $view->role);
    }
    if ($view->data) {
        $c->stash(data => { %{ $view->data }, put__role => $c->stash->{role} });
    }
    if ($view->redirect) {
        $c->res->redirect($view->redirect);
    }
}

sub error404 : Action {
    my ($self, $c) = @_;
    $c->stash(template => 'error404.tt');
    $c->res->code(404);
}

sub error403 : Action {
    my ($self, $c) = @_;
    $c->stash(template => 'error403.tt');
    $c->res->code(403);
}

sub end : ActionClass('RenderView') {
    my ($self, $c) = @_;
}

1;

=head1 NAME

CatalystX::state51::LegacySupport - 

=head1 DESCRIPTION

=head1 METHODS

=head1 BUGS

=head1 AUTHOR

=head1 COPYRIGHT & LICENSE

Copyright 2009 the above author(s).

=cut

